<!DOCTYPE html>
<html>

  <head>
    <title>{{ title }}</title>
    <meta http-equiv="Content-Type" content="text/html;charset=utf-8">
    <base target="_top">
    <meta name="robots" content="noindex,nofollow">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="Pragma" content="no-cache">
    <meta http-equiv="x-frame-options" content="deny">
    <meta name="Cache-Control" content="private">
    <link
      rel="stylesheet"
      type="text/css"
      href="/css/ae-dir.css"
      integrity="sha512-siysDlkfJaVSRPFYCnV4aYq2JXPJ2+X2nRVypp+ELXKc53dKcHTHQImojwRn1VURAg7LGgQG81FQv3QD8SGflQ=="
    >
    <link
      href="/css/ae-web-app.css"
      rel="stylesheet"
      type="text/css"
      integrity="sha512-uwYzSwoFbj9LoTE939wqbi/Kbf23/N6hheXHzZ/wnY3X7a3woFF3S+Lj0dtED/JV4t2gVKlkQ/Y5aeA6IOtv6A=="
    >
  </head>

  <body>

    <div id="navigation">
      <a title="Overview of OATH-LDAP Token Enrollment Service" href=".">OATH-LDAP</a> /
      <a title="Reset OATH token and request enrollment password" href="reset">Reset</a>
    </div>

<div id="content">
    {% block content %}
    {% endblock content %}
</div>

    <p id="footer">
      <a href="/oath.html"
         title="Read user guide for OATH-LDAP Token Enrollment Service">User guide</a>
      //
      <a href="/web2ldap"
         title="Access to web2ldap application">web2ldap</a>
      //
      <a href="/pwd/"
         title="Access to &AElig;-DIR password self-service">Password self-service</a>
    </p>

  </body>

</html>
